/*
 * Project name:
     RTC2_Read (Reading date/time data from DS1307 through I2C)
 * Copyright:
     (c) MikroElektronika, 2010
 * Revision History:
     20091212:
        - initial release;
* Description:
     This project is simple demonstration how to read date and time from DS1307
     RTC (real-time clock). The code MCU use TWI module at PORTD.
     Date and time are printed at LCD.
 * Test configuration:
     MCU:             ATmega128
                      http://www.atmel.com/dyn/resources/prod_documents/doc2467.pdf
     Dev.Board:       BIGAVR6 - ac:RTC
                      http://www.mikroe.com/bigavr/
     Oscillator:      External Clock 08.0000 MHz
     Ext. Modules:    LCD 2x16 module
                      http://www.mikroe.com/store/components/various/
     SW:              mikroC PRO for AVR
                      http://www.mikroe.com/mikroc/avr/
 * NOTES:
     - Turn on I2C lines SW14.5 and SW14.7
     - For proper I2C communication, pins on PORTD must be in the pull-up mode,
       and the LEDs on board switched OFF!
*/

unsigned char sec, min1, hr, week_day, day, mn, year;
char *txt, tnum[4];

// LCD module connections
sbit LCD_RS at PORTC2_bit;
sbit LCD_EN at PORTC3_bit;
sbit LCD_D4 at PORTC4_bit;
sbit LCD_D5 at PORTC5_bit;
sbit LCD_D6 at PORTC6_bit;
sbit LCD_D7 at PORTC7_bit;

sbit LCD_RS_Direction at DDC2_bit;
sbit LCD_EN_Direction at DDC3_bit;
sbit LCD_D4_Direction at DDC4_bit;
sbit LCD_D5_Direction at DDC5_bit;
sbit LCD_D6_Direction at DDC6_bit;
sbit LCD_D7_Direction at DDC7_bit;
// End LCD module connections

void Zero_Fill(char *value) {           // fill text repesentation
  if (value[1] == 0) {                  // with leading zero
    value[1] = value[0];
    value[0] = 48;
    value[2] = 0;
  }
}//~

//--------------------- Reads time and date information from RTC (DS1307)
void Read_Time(char *sec, char *min, char *hr, char *week_day, char *day, char *mn, char *year) {
  TWI_Start();
  TWI_Write(0xD0);
  TWI_Write(0);
  TWI_Stop();
  TWI_Start();

  TWI_Write(0xD1);
  *sec =TWI_Read(1);
  *min =TWI_Read(1);
  *hr =TWI_Read(1);
  *week_day =TWI_Read(1);
  *day =TWI_Read(1);
  *mn =TWI_Read(1);
  *year =TWI_Read(0);
  TWI_Stop();
}//~

//-------------------- Formats date and time
void Transform_Time(char  *sec, char *min, char *hr, char *week_day, char *day, char *mn, char *year) {
  *sec  =  ((*sec & 0x70) >> 4)*10 + (*sec & 0x0F);
  *min  =  ((*min & 0xF0) >> 4)*10 + (*min & 0x0F);
  *hr   =  ((*hr & 0x30) >> 4)*10 + (*hr & 0x0F);
  *week_day =(*week_day & 0x07);
  *day  =  ((*day & 0xF0) >> 4)*10 + (*day & 0x0F);
  *mn   =  ((*mn & 0x10) >> 4)*10 + (*mn & 0x0F);
  *year =  ((*year & 0xF0)>>4)*10+(*year & 0x0F);
}//~

//-------------------- Output values to LCD
void Display_Time(char sec, char min, char hr, char week_day, char day, char mn, char year) {
   switch(week_day){
     case 1: txt="Sun"; break;
     case 2: txt="Mon"; break;
     case 3: txt="Tue"; break;
     case 4: txt="Wed"; break;
     case 5: txt="Thu"; break;
     case 6: txt="Fri"; break;
     case 7: txt="Sat"; break;
   }
   Lcd_Out(1,1,txt);
   Lcd_Chr(1, 6, (day / 10)   + 48);    // Print tens digit of day variable
   Lcd_Chr(1, 7, (day % 10)   + 48);    // Print ones digit of day variable
   Lcd_Chr(1, 9, (mn / 10) + 48);
   Lcd_Chr(1,10, (mn % 10) + 48);
   Lcd_Chr(1,14, (year / 10) + 48);    // Print year vaiable2009)
   Lcd_Chr(1,15, (year % 10) + 48);

   Lcd_Chr(2, 6, (hr / 10)   + 48);
   Lcd_Chr(2, 7, (hr % 10)   + 48);
   Lcd_Chr(2, 9, (min / 10) + 48);
   Lcd_Chr(2,10, (min % 10) + 48);
   Lcd_Chr(2,12, (sec / 10) + 48);
   Lcd_Chr(2,13, (sec % 10) + 48);
   
}//~

//------------------ Performs project-wide init
void Init_Main() {

  Lcd_Init();                           // Initialize LCD
  Lcd_Cmd(_LCD_CLEAR);                  // Clear LCD display
  Lcd_Cmd(_LCD_CURSOR_OFF);             // Turn cursor off

  TWI_Init(100000);                     // initialize I2C
  Lcd_Chr(1,8,'.');
  Lcd_Chr(1,11,'.');
  txt = "Time:";
  Lcd_Out(2,1,txt);
  Lcd_Chr(2,8,':');
  Lcd_Chr(2,11,':');
  txt = "2000";
  Lcd_Out(1,12,txt);
  Lcd_Cmd(_LCD_CURSOR_OFF);
}//~

//----------------- Main procedure
void main() {
  Init_Main();                                               // Perform initialization
  while (1) {                                                // Endless loop
    Read_Time(&sec,&min1,&hr,&week_day,&day,&mn,&year);      // read time from RTC(DS1307)
    Transform_Time(&sec,&min1,&hr,&week_day,&day,&mn,&year); // format date and time
    Display_Time(sec, min1, hr, week_day, day, mn, year);    // prepare and display on LCD
    Delay_ms(1000);                                          // wait 1s
  }
}//