/*
 * Project name:
     CFlash_Common (Basic use of Compact Flash common library)
 * Copyright:
     (c) Mikroelektronika, 2005-2010.
 * Revision History:
     20080930:
       - initial release;
 * Description:
     This project is a simple demonstration of CF(Compact Flash) Library which
     shows how to use CF card data accessing routines. With minor adjustments,
     this example should work with any other MCU.
     Data access routines need two ports for communication with CF card.
     Before write operation make sure that you do not overwrite boot or FAT
     sector, because it could make your card unreadable by a PC or by a digital
     photo camera.
 * Test configuration:
     MCU:             ATmega16
                      http://www.atmel.com/dyn/resources/prod_documents/doc2466.pdf
     Dev.Board:       EasyAVR6
                      http://www.mikroe.com/easyavr/v6/
     Oscillator:      External, 8.0000 MHz
     Ext. Modules:    ac:Compact_Flash_Board on PORTB(Control) and PORTD(Data)
     SW:              mikroC PRO for AVR 
                      http://www.mikroe.com/mikroc/avr/
 * NOTES:
     - This example does NOT cover writing/reading of FILES on the CF media. This
       topic is covered in CF_FAT16 examples.
     - For some CF cards external power supply may be required.
 */

// set compact flash pinout
char Cf_Data_Port at PORTD;
char Cf_Data_Port_Direction at DDRD;

sbit CF_RDY at PINB7_bit;
sbit CF_WE  at PORTB6_bit;
sbit CF_OE  at PORTB5_bit;
sbit CF_CD1 at PINB4_bit;
sbit CF_CE1 at PORTB3_bit;
sbit CF_A2  at PORTB2_bit;
sbit CF_A1  at PORTB1_bit;
sbit CF_A0  at PORTB0_bit;

sbit CF_RDY_direction at DDB7_bit;
sbit CF_WE_direction  at DDB6_bit;
sbit CF_OE_direction  at DDB5_bit;
sbit CF_CD1_direction at DDB4_bit;
sbit CF_CE1_direction at DDB3_bit;
sbit CF_A2_direction  at DDB2_bit;
sbit CF_A1_direction  at DDB1_bit;
sbit CF_A0_direction  at DDB0_bit;
// end of cf pinout

char SignalPort           at PORTC;
char SignalPort_direction at DDRC;

void InitCF() {
  CF_CD1_direction = 0;                   // set CD1 pin as input
  while (Cf_Detect() == 0)                // wait until CF card is inserted
    ;
  Cf_Init();                              // initialize CF
  while (!CF_RDY)
    ;
  Delay_ms(2000);                         // wait for a while until the card is stabilized
}                                         //   period depends on used CF card

void TestBytes() {
  unsigned int i;

  ///// Write numbers 0..511 to sector 590
  Cf_Write_Init(590,1);                   // Initialize write at sector address 590
                                          //     for 1 sector
  SignalPort = 0x03;                      // Notify that write has started
  Delay_ms(1000);
  for (i=0; i<=511; i++)                  // Write 512 bytes to sector 590
    Cf_Write_Byte(i);

  SignalPort = 0x07;                      // Notify that write end and read start
  Delay_ms(1000);
  
  Cf_Read_Init(590,1);                    // Initialize read from sector address 590
                                          //    for 1 sector
  for (i=0; i<=511; i++) {                // Read 512 bytes from sector address 590
    SignalPort = Cf_Read_Byte();          // Read one byte at time and display
                                          //    readings on signal port
    Delay_ms(5);                          // Wait for a while to see results
  }
  Delay_ms(1000);

  ///// Write numbers 511..0 to sector 590
  Cf_Write_Init(590,1);                   // Initialize write at sector address 590
                                          //     for 1 sector
  SignalPort = 0x03;                      // Notify that write has started
  Delay_ms(1000);
  for (i=0; i<=511; i++)                  // Write 512 bytes to sector 590
    Cf_Write_Byte(511-i);

  SignalPort = 0x07;                      // Notify that write end and read start
  Delay_ms(1000);

  Cf_Read_Init(590,1);                    // Initialize read from sector address 590
                                          //    for 1 sector
  for (i=0; i<=511; i++) {                // Read 512 bytes from sector address 590
    SignalPort = Cf_Read_Byte();          // Read one byte at time and display
                                          //    readings on signal port
    Delay_ms(5);                          // Wait for a while to see results
  }
  Delay_ms(1000);
}

// Main program
void main() {
 
  SignalPort_direction = 0xFF;            // designate signal port as output
  SignalPort = 0x01;                      // Notify test start
  InitCF();

  TestBytes();
  SignalPort = 0x0F;                      // Notify test end
}