/*
 * Project name:
     SPI (Driving external Digital-to-Analog converter)
 * Copyright:
     (c) Mikroelektronika, 2005-2010.
 * Revision History:
     20080930:
       - initial release;
 * Description:
     This is a sample program which demonstrates the use of the Microchip's
     MCP4921 12-bit D/A converter with AVR MCUs. This device accepts digital
     input (number from 0..4095) and transforms it to the output voltage,
     ranging from 0..Vref. In this example the DAC communicates with MCU
     through the SPI communication.
     Buttons PA0 and PA1 are used to change value sent to DAC.
 * Test configuration:
     MCU:             ATmega16
                      http://www.atmel.com/dyn/resources/prod_documents/doc2466.pdf
     Dev.Board:       EasyAVR6
                      http://www.mikroe.com/easyavr/v6/
     Oscillator:      External Clock, 8.0000 MHz
     Ext. Modules:    ac:DAC_Board on PORTB
     SW:              mikroC PRO for AVR
                      http://www.mikroe.com/mikroc/avr/
 * NOTES:
     - Turn on AVR and CS0 switches on DAC extra board.
     - Put button jumper (J13) into VCC position and pull-down PORTA. (board specific)
 */

// DAC module connections
sbit Chip_Select at PORTB1_bit;
sbit Chip_Select_Direction at DDB1_bit;
// End DAC module connections

unsigned int value;                     

void InitMain() {
  DDA0_bit = 0;                           // Set PA0 pin as input
  DDA1_bit = 0;                           // Set PA1 pin as input
  Chip_Select = 1;                        // Deselect DAC
  Chip_Select_Direction = 1;              // Set CS# pin as Output
  SPI1_Init();                            // Initialize SPI1 module
}

// DAC increments (0..4095) --> output voltage (0..Vref)
void DAC_Output(unsigned int valueDAC) {
  char temp;
 
  Chip_Select = 0;                        // Select DAC chip
  
  // Send High Byte                                         
  temp = (valueDAC >> 8) & 0x0F;          // Store valueDAC[11..8] to temp[3..0]
  temp |= 0x30;                           // Define DAC setting, see MCP4921 datasheet
  SPI1_Write(temp);                       // Send high byte via SPI
  
  // Send Low Byte
  temp = valueDAC;                        // Store valueDAC[7..0] to temp[7..0]
  SPI1_Write(temp);                       // Send low byte via SPI
  
  Chip_Select = 1;                        // Deselect DAC chip
}

void main() {

  InitMain();                             // Perform main initialization

  value = 2048;                           // When program starts, DAC gives
                                          //   the output in the mid-range
                                          
  while (1) {                             // Endless loop

    if ((PINA0_bit) && (value < 4095)) {  // If PA0 button is pressed
      value++;                            //   increment value
      }
    else {
      if ((PINA1_bit) && (value > 0)) {   // If PA1 button is pressed
        value--;                          //   decrement value
        }
      }

    DAC_Output(value);                    // Send value to DAC chip
    Delay_ms(1);                          // Slow down key repeat pace
  }
}